#include <windows.h>
#include <string.h>
#include <io.h> 
#include <wchar.h>

const int CDIRSize = 1024;
const int WorkSize = CDIRSize * 2;
char      CurrentDir[CDIRSize];
char      WorkBuffer[WorkSize];

// ---------------------------------------------------------------------------------------------------------------------------------

// Function converts a lower case string into upper case, any special 
// characters remain the same (i.e "$", "%" ...)
void StrUpr (char* pUpCase)
{
     while (*pUpCase != '\0')
     {
          if ((*pUpCase >= 'a') && (*pUpCase <= 'z'))
                  *pUpCase = (*pUpCase & 223);
          pUpCase++;
     }
}

// ---------------------------------------------------------------------------------------------------------------------------------

char* GetDesktopBGString (HKEY hDesktopBG)
{
    DWORD          paperLen  = 0;
    unsigned long  type      = 0;
    char*          pPaperMem = NULL;

    // Querry registry for wallpaper size.
    RegQueryValueEx(hDesktopBG, (LPCTSTR)"Wallpaper", NULL, &type, NULL, &paperLen);

    if (paperLen <= 0)
        return NULL;

    // allocate mem for wallpaper string
    if ((pPaperMem = new char[paperLen]) == NULL)
        return NULL;

    // Querry registry for wallpaper size.
    RegQueryValueEx(hDesktopBG, (LPCTSTR)"Wallpaper", NULL, &type, (LPBYTE)pPaperMem, &paperLen);
    
    return pPaperMem;
}

// ---------------------------------------------------------------------------------------------------------------------------------

// returns :
// -1 : not bmp file in dir
int GetNextFile(char* pCurWallpaper, char* pCurrentDir, char* pReturnBuffer)
{
    _finddata_t     fileInfo;
    char*           fileSpec   = "*.bmp";
    __int64         hFileInfo  = 0;
    int             moreFiles  = 0;

    //findRtnVal = _wfindfirsti64(fileSpec, &fileInfo);
    hFileInfo = _findfirst(fileSpec, &fileInfo);


    if (hFileInfo < 0)
        return -1;

    // upcase both file names 
    StrUpr (pCurrentDir);
    StrUpr (pCurWallpaper);
    StrUpr ((char*)fileInfo.name);

    // Construct string so they are constructed the same for comparing
    strcpy (pReturnBuffer, pCurrentDir);
    strcat (pReturnBuffer, "\\");
    strcat (pReturnBuffer, (char*)fileInfo.name);
    

    // Find the original file ... in the directory
    while ((strcmp (pCurWallpaper, pReturnBuffer) != 0) && (moreFiles == 0))
    {
        // Get next file to test
        moreFiles = _findnext((long)hFileInfo, &fileInfo);
        StrUpr ((char*)fileInfo.name);

        strcpy (pReturnBuffer, pCurrentDir);
        strcat (pReturnBuffer, "\\");
        strcat (pReturnBuffer, (char*)fileInfo.name);
    }
    
    if (moreFiles != 0)                             // No Match found !
    {
        // close the current file handle 
        _findclose((long) hFileInfo);

        // Use the first file in the directory!
        hFileInfo = _findfirst(fileSpec, &fileInfo);
    }
    else                                           // found a match ...
    {
        // use the next file in the directory
        if ((_findnext((long)hFileInfo, &fileInfo)) != 0)
        {
             // close the current file handle 
            _findclose((long) hFileInfo);

            // opps, no more next file ... use the first file!
            hFileInfo = _findfirst(fileSpec, &fileInfo);
        }
    }

    // close the current file handle 
    _findclose((long) hFileInfo);

    // construct final wallpaper string!
    strcpy (pReturnBuffer, pCurrentDir);
    strcat (pReturnBuffer, "\\");
    strcat (pReturnBuffer, (char*)fileInfo.name);

    return 0;
}

// ---------------------------------------------------------------------------------------------------------------------------------

BOOL getcmdParam (LPSTR pCMD, int& sleepTime)
{
    int time = atoi (pCMD);
    
    if (time > 0) 
        sleepTime = ((time * 1000) * 60);
    else
        return FALSE;

    return TRUE;
}

// ---------------------------------------------------------------------------------------------------------------------------------


int PASCAL    WinMain( HANDLE hInstance,        // Instance handle 
                       HANDLE hPrevInstance,    // Previous instance handle 
                       LPSTR lpszCommandLine,   // Command line string 
                       int cmdShow )            // ShowWindow flag 
{ 

    int  sleepTime = 0;
    BOOL loop      = getcmdParam (lpszCommandLine, sleepTime);

    do
    {
        HKEY  hDesktopBG = NULL;
        LONG  result     = RegOpenKey (HKEY_CURRENT_USER, (LPCTSTR)"Control Panel\\Desktop", &hDesktopBG);
        char* pWallPaper = NULL;  
    
        if (result != 0)
            return -1;

        pWallPaper = GetDesktopBGString (hDesktopBG);

        RegCloseKey(hDesktopBG); 

        if (pWallPaper != NULL)
        {
            GetCurrentDirectory ((DWORD)CDIRSize, (LPTSTR) CurrentDir);
                    
            if (GetNextFile(pWallPaper, CurrentDir, WorkBuffer) != 0)
                MessageBox (NULL, "Unable to find BMP files!", "No Files Found Error", MB_ICONSTOP | MB_OK);
            else
            {
                if (SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (PVOID)WorkBuffer, SPIF_UPDATEINIFILE) == FALSE)
                    MessageBox (NULL, "Unable to set new background ", "Update Error", MB_ICONSTOP | MB_OK);
            }
        }
   
        delete[] pWallPaper;    

        Sleep (sleepTime);

    } while (loop != FALSE);

    return 0;
} 
